"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var BaseMethods_js_1 = __importDefault(require("../base/BaseMethods.js"));
var MmlNode_js_1 = require("../../../core/MmlTree/MmlNode.js");
var TexError_js_1 = __importDefault(require("../TexError.js"));
var BraketMethods = {};
BraketMethods.Macro = BaseMethods_js_1.default.Macro;
BraketMethods.Braket = function (parser, _name, open, close, stretchy, barmax, space) {
    if (space === void 0) { space = false; }
    var next = parser.GetNext();
    if (next === '') {
        throw new TexError_js_1.default('MissingArgFor', 'Missing argument for %1', parser.currentCS);
    }
    var single = true;
    if (next === '{') {
        parser.i++;
        single = false;
    }
    parser.Push(parser.itemFactory.create('braket')
        .setProperties({ barcount: 0, barmax: barmax, open: open, close: close, stretchy: stretchy, single: single, space: space }));
};
BraketMethods.Bar = function (parser, name) {
    var c = name === '|' ? '|' : '\u2016';
    var top = parser.stack.Top();
    if (top.isKind('over')) {
        top = parser.stack.Top(2);
    }
    if (!top.isKind('braket') || top.getProperty('barcount') >= top.getProperty('barmax')) {
        return false;
    }
    if (c === '|' && parser.GetNext() === '|') {
        parser.i++;
        c = '\u2016';
    }
    var stretchy = top.getProperty('stretchy');
    if (!stretchy) {
        var node = parser.create('token', 'mo', { stretchy: false, 'data-braketbar': true, texClass: MmlNode_js_1.TEXCLASS.ORD }, c);
        parser.Push(node);
        return;
    }
    var close = parser.itemFactory.create('close').setProperty('braketbar', true);
    parser.Push(close);
    top.barNodes.push(parser.create('node', 'TeXAtom', [], { texClass: MmlNode_js_1.TEXCLASS.CLOSE }), parser.create('token', 'mo', { stretchy: true, 'data-braketbar': true, texClass: MmlNode_js_1.TEXCLASS.BIN }, c), parser.create('node', 'TeXAtom', [], { texClass: MmlNode_js_1.TEXCLASS.OPEN }));
    top.setProperty('barcount', top.getProperty('barcount') + 1);
};
exports.default = BraketMethods;
//# sourceMappingURL=BraketMethods.js.map