import * as sm from '../TokenMap.js';
import { TexConstant } from '../TexConstants.js';
import BaseMethods from './BaseMethods.js';
import ParseMethods from '../ParseMethods.js';
import ParseUtil from '../ParseUtil.js';
import { TEXCLASS } from '../../../core/MmlTree/MmlNode.js';
import { MATHSPACE, em } from '../../../util/lengths.js';
const THICKMATHSPACE = em(MATHSPACE.thickmathspace);
const VARIANT = TexConstant.Variant;
new sm.RegExpMap('letter', ParseMethods.variable, /[a-z]/i);
new sm.RegExpMap('digit', ParseMethods.digit, /[0-9.,]/);
new sm.RegExpMap('command', ParseMethods.controlSequence, /^\\/);
new sm.MacroMap('special', {
    '{': 'Open',
    '}': 'Close',
    '~': 'Tilde',
    '^': 'Superscript',
    '_': 'Subscript',
    '|': 'Bar',
    ' ': 'Space',
    '\t': 'Space',
    '\r': 'Space',
    '\n': 'Space',
    '\'': 'Prime',
    '%': 'Comment',
    '&': 'Entry',
    '#': 'Hash',
    '\u00A0': 'Space',
    '\u2019': 'Prime'
}, BaseMethods);
new sm.CharacterMap('lcGreek', ParseMethods.lcGreek, {
    alpha: '\u03B1',
    beta: '\u03B2',
    gamma: '\u03B3',
    delta: '\u03B4',
    epsilon: '\u03F5',
    zeta: '\u03B6',
    eta: '\u03B7',
    theta: '\u03B8',
    iota: '\u03B9',
    kappa: '\u03BA',
    lambda: '\u03BB',
    mu: '\u03BC',
    nu: '\u03BD',
    xi: '\u03BE',
    omicron: '\u03BF',
    pi: '\u03C0',
    rho: '\u03C1',
    sigma: '\u03C3',
    tau: '\u03C4',
    upsilon: '\u03C5',
    phi: '\u03D5',
    chi: '\u03C7',
    psi: '\u03C8',
    omega: '\u03C9',
    varepsilon: '\u03B5',
    vartheta: '\u03D1',
    varpi: '\u03D6',
    varrho: '\u03F1',
    varsigma: '\u03C2',
    varphi: '\u03C6'
});
new sm.CharacterMap('ucGreek', ParseMethods.ucGreek, {
    Gamma: '\u0393',
    Delta: '\u0394',
    Theta: '\u0398',
    Lambda: '\u039B',
    Xi: '\u039E',
    Pi: '\u03A0',
    Sigma: '\u03A3',
    Upsilon: '\u03A5',
    Phi: '\u03A6',
    Psi: '\u03A8',
    Omega: '\u03A9'
});
new sm.CharacterMap('mathchar0mi', ParseMethods.mathchar0mi, {
    AA: '\u212B',
    S: ['\u00A7', { mathvariant: VARIANT.NORMAL }],
    aleph: ['\u2135', { mathvariant: VARIANT.NORMAL }],
    hbar: ['\u210F', { variantForm: true }],
    imath: '\u0131',
    jmath: '\u0237',
    ell: '\u2113',
    wp: ['\u2118', { mathvariant: VARIANT.NORMAL }],
    Re: ['\u211C', { mathvariant: VARIANT.NORMAL }],
    Im: ['\u2111', { mathvariant: VARIANT.NORMAL }],
    partial: ['\u2202', { mathvariant: VARIANT.ITALIC }],
    infty: ['\u221E', { mathvariant: VARIANT.NORMAL }],
    prime: ['\u2032', { variantForm: true }],
    emptyset: ['\u2205', { mathvariant: VARIANT.NORMAL }],
    nabla: ['\u2207', { mathvariant: VARIANT.NORMAL }],
    top: ['\u22A4', { mathvariant: VARIANT.NORMAL }],
    bot: ['\u22A5', { mathvariant: VARIANT.NORMAL }],
    angle: ['\u2220', { mathvariant: VARIANT.NORMAL }],
    triangle: ['\u25B3', { mathvariant: VARIANT.NORMAL }],
    backslash: ['\\', { mathvariant: VARIANT.NORMAL }],
    forall: ['\u2200', { mathvariant: VARIANT.NORMAL }],
    exists: ['\u2203', { mathvariant: VARIANT.NORMAL }],
    neg: ['\u00AC', { mathvariant: VARIANT.NORMAL }],
    lnot: ['\u00AC', { mathvariant: VARIANT.NORMAL }],
    flat: ['\u266D', { mathvariant: VARIANT.NORMAL }],
    natural: ['\u266E', { mathvariant: VARIANT.NORMAL }],
    sharp: ['\u266F', { mathvariant: VARIANT.NORMAL }],
    clubsuit: ['\u2663', { mathvariant: VARIANT.NORMAL }],
    diamondsuit: ['\u2662', { mathvariant: VARIANT.NORMAL }],
    heartsuit: ['\u2661', { mathvariant: VARIANT.NORMAL }],
    spadesuit: ['\u2660', { mathvariant: VARIANT.NORMAL }]
});
new sm.CharacterMap('mathchar0mo', ParseMethods.mathchar0mo, {
    surd: '\u221A',
    coprod: ['\u2210', { movesupsub: true }],
    bigvee: ['\u22C1', { movesupsub: true }],
    bigwedge: ['\u22C0', { movesupsub: true }],
    biguplus: ['\u2A04', { movesupsub: true }],
    bigcap: ['\u22C2', { movesupsub: true }],
    bigcup: ['\u22C3', { movesupsub: true }],
    'int': '\u222B',
    intop: ['\u222B', { movesupsub: true, movablelimits: true }],
    iint: '\u222C',
    iiint: '\u222D',
    prod: ['\u220F', { movesupsub: true }],
    sum: ['\u2211', { movesupsub: true }],
    bigotimes: ['\u2A02', { movesupsub: true }],
    bigoplus: ['\u2A01', { movesupsub: true }],
    bigodot: ['\u2A00', { movesupsub: true }],
    oint: '\u222E',
    ointop: ['\u222E', { movesupsub: true, movablelimits: true }],
    oiint: '\u222F',
    oiiint: '\u2230',
    bigsqcup: ['\u2A06', { movesupsub: true }],
    smallint: ['\u222B', { largeop: false }],
    triangleleft: '\u25C3',
    triangleright: '\u25B9',
    bigtriangleup: '\u25B3',
    bigtriangledown: '\u25BD',
    wedge: '\u2227',
    land: '\u2227',
    vee: '\u2228',
    lor: '\u2228',
    cap: '\u2229',
    cup: '\u222A',
    ddagger: '\u2021',
    dagger: '\u2020',
    sqcap: '\u2293',
    sqcup: '\u2294',
    uplus: '\u228E',
    amalg: '\u2A3F',
    diamond: '\u22C4',
    bullet: '\u2219',
    wr: '\u2240',
    div: '\u00F7',
    divsymbol: '\u00F7',
    odot: ['\u2299', { largeop: false }],
    oslash: ['\u2298', { largeop: false }],
    otimes: ['\u2297', { largeop: false }],
    ominus: ['\u2296', { largeop: false }],
    oplus: ['\u2295', { largeop: false }],
    mp: '\u2213',
    pm: '\u00B1',
    circ: '\u2218',
    bigcirc: '\u25EF',
    setminus: '\u2216',
    cdot: '\u22C5',
    ast: '\u2217',
    times: '\u00D7',
    star: '\u22C6',
    propto: '\u221D',
    sqsubseteq: '\u2291',
    sqsupseteq: '\u2292',
    parallel: '\u2225',
    mid: '\u2223',
    dashv: '\u22A3',
    vdash: '\u22A2',
    leq: '\u2264',
    le: '\u2264',
    geq: '\u2265',
    ge: '\u2265',
    lt: '\u003C',
    gt: '\u003E',
    succ: '\u227B',
    prec: '\u227A',
    approx: '\u2248',
    succeq: '\u2AB0',
    preceq: '\u2AAF',
    supset: '\u2283',
    subset: '\u2282',
    supseteq: '\u2287',
    subseteq: '\u2286',
    'in': '\u2208',
    ni: '\u220B',
    notin: '\u2209',
    owns: '\u220B',
    gg: '\u226B',
    ll: '\u226A',
    sim: '\u223C',
    simeq: '\u2243',
    perp: '\u22A5',
    equiv: '\u2261',
    asymp: '\u224D',
    smile: '\u2323',
    frown: '\u2322',
    ne: '\u2260',
    neq: '\u2260',
    cong: '\u2245',
    doteq: '\u2250',
    bowtie: '\u22C8',
    models: '\u22A7',
    notChar: '\u29F8',
    Leftrightarrow: '\u21D4',
    Leftarrow: '\u21D0',
    Rightarrow: '\u21D2',
    leftrightarrow: '\u2194',
    leftarrow: '\u2190',
    gets: '\u2190',
    rightarrow: '\u2192',
    to: ['\u2192', { accent: false }],
    mapsto: '\u21A6',
    leftharpoonup: '\u21BC',
    leftharpoondown: '\u21BD',
    rightharpoonup: '\u21C0',
    rightharpoondown: '\u21C1',
    nearrow: '\u2197',
    searrow: '\u2198',
    nwarrow: '\u2196',
    swarrow: '\u2199',
    rightleftharpoons: '\u21CC',
    hookrightarrow: '\u21AA',
    hookleftarrow: '\u21A9',
    longleftarrow: '\u27F5',
    Longleftarrow: '\u27F8',
    longrightarrow: '\u27F6',
    Longrightarrow: '\u27F9',
    Longleftrightarrow: '\u27FA',
    longleftrightarrow: '\u27F7',
    longmapsto: '\u27FC',
    ldots: '\u2026',
    cdots: '\u22EF',
    vdots: '\u22EE',
    ddots: '\u22F1',
    iddots: '\u22F0',
    dotsc: '\u2026',
    dotsb: '\u22EF',
    dotsm: '\u22EF',
    dotsi: '\u22EF',
    dotso: '\u2026',
    ldotp: ['\u002E', { texClass: TEXCLASS.PUNCT }],
    cdotp: ['\u22C5', { texClass: TEXCLASS.PUNCT }],
    colon: ['\u003A', { texClass: TEXCLASS.PUNCT }]
});
new sm.CharacterMap('mathchar7', ParseMethods.mathchar7, {
    '_': '\u005F',
    '#': '\u0023',
    '$': '\u0024',
    '%': '\u0025',
    '&': '\u0026',
    And: '\u0026'
});
new sm.DelimiterMap('delimiter', ParseMethods.delimiter, {
    '(': '(',
    ')': ')',
    '[': '[',
    ']': ']',
    '<': '\u27E8',
    '>': '\u27E9',
    '\\lt': '\u27E8',
    '\\gt': '\u27E9',
    '/': '/',
    '|': ['|', { texClass: TEXCLASS.ORD }],
    '.': '',
    '\\\\': '\\',
    '\\lmoustache': '\u23B0',
    '\\rmoustache': '\u23B1',
    '\\lgroup': '\u27EE',
    '\\rgroup': '\u27EF',
    '\\arrowvert': '\u23D0',
    '\\Arrowvert': '\u2016',
    '\\bracevert': '\u23AA',
    '\\Vert': ['\u2016', { texClass: TEXCLASS.ORD }],
    '\\|': ['\u2016', { texClass: TEXCLASS.ORD }],
    '\\vert': ['|', { texClass: TEXCLASS.ORD }],
    '\\uparrow': '\u2191',
    '\\downarrow': '\u2193',
    '\\updownarrow': '\u2195',
    '\\Uparrow': '\u21D1',
    '\\Downarrow': '\u21D3',
    '\\Updownarrow': '\u21D5',
    '\\backslash': '\\',
    '\\rangle': '\u27E9',
    '\\langle': '\u27E8',
    '\\rbrace': '}',
    '\\lbrace': '{',
    '\\}': '}',
    '\\{': '{',
    '\\rceil': '\u2309',
    '\\lceil': '\u2308',
    '\\rfloor': '\u230B',
    '\\lfloor': '\u230A',
    '\\lbrack': '[',
    '\\rbrack': ']'
});
new sm.CommandMap('macros', {
    displaystyle: ['SetStyle', 'D', true, 0],
    textstyle: ['SetStyle', 'T', false, 0],
    scriptstyle: ['SetStyle', 'S', false, 1],
    scriptscriptstyle: ['SetStyle', 'SS', false, 2],
    rm: ['SetFont', VARIANT.NORMAL],
    mit: ['SetFont', VARIANT.ITALIC],
    oldstyle: ['SetFont', VARIANT.OLDSTYLE],
    cal: ['SetFont', VARIANT.CALLIGRAPHIC],
    it: ['SetFont', VARIANT.MATHITALIC],
    bf: ['SetFont', VARIANT.BOLD],
    sf: ['SetFont', VARIANT.SANSSERIF],
    tt: ['SetFont', VARIANT.MONOSPACE],
    frak: ['MathFont', VARIANT.FRAKTUR],
    Bbb: ['MathFont', VARIANT.DOUBLESTRUCK],
    mathrm: ['MathFont', VARIANT.NORMAL],
    mathup: ['MathFont', VARIANT.NORMAL],
    mathnormal: ['MathFont', ''],
    mathbf: ['MathFont', VARIANT.BOLD],
    mathbfup: ['MathFont', VARIANT.BOLD],
    mathit: ['MathFont', VARIANT.MATHITALIC],
    mathbfit: ['MathFont', VARIANT.BOLDITALIC],
    mathbb: ['MathFont', VARIANT.DOUBLESTRUCK],
    mathfrak: ['MathFont', VARIANT.FRAKTUR],
    mathbffrak: ['MathFont', VARIANT.BOLDFRAKTUR],
    mathscr: ['MathFont', VARIANT.SCRIPT],
    mathbfscr: ['MathFont', VARIANT.BOLDSCRIPT],
    mathsf: ['MathFont', VARIANT.SANSSERIF],
    mathsfup: ['MathFont', VARIANT.SANSSERIF],
    mathbfsf: ['MathFont', VARIANT.BOLDSANSSERIF],
    mathbfsfup: ['MathFont', VARIANT.BOLDSANSSERIF],
    mathsfit: ['MathFont', VARIANT.SANSSERIFITALIC],
    mathbfsfit: ['MathFont', VARIANT.SANSSERIFBOLDITALIC],
    mathtt: ['MathFont', VARIANT.MONOSPACE],
    mathcal: ['MathFont', VARIANT.CALLIGRAPHIC],
    mathbfcal: ['MathFont', VARIANT.BOLDCALLIGRAPHIC],
    symrm: ['MathFont', VARIANT.NORMAL],
    symup: ['MathFont', VARIANT.NORMAL],
    symnormal: ['MathFont', ''],
    symbf: ['MathFont', VARIANT.BOLD],
    symbfup: ['MathFont', VARIANT.BOLD],
    symit: ['MathFont', VARIANT.ITALIC],
    symbfit: ['MathFont', VARIANT.BOLDITALIC],
    symbb: ['MathFont', VARIANT.DOUBLESTRUCK],
    symfrak: ['MathFont', VARIANT.FRAKTUR],
    symbffrak: ['MathFont', VARIANT.BOLDFRAKTUR],
    symscr: ['MathFont', VARIANT.SCRIPT],
    symbfscr: ['MathFont', VARIANT.BOLDSCRIPT],
    symsf: ['MathFont', VARIANT.SANSSERIF],
    symsfup: ['MathFont', VARIANT.SANSSERIF],
    symbfsf: ['MathFont', VARIANT.BOLDSANSSERIF],
    symbfsfup: ['MathFont', VARIANT.BOLDSANSSERIF],
    symsfit: ['MathFont', VARIANT.SANSSERIFITALIC],
    symbfsfit: ['MathFont', VARIANT.SANSSERIFBOLDITALIC],
    symtt: ['MathFont', VARIANT.MONOSPACE],
    symcal: ['MathFont', VARIANT.CALLIGRAPHIC],
    symbfcal: ['MathFont', VARIANT.BOLDCALLIGRAPHIC],
    textrm: ['HBox', null, VARIANT.NORMAL],
    textup: ['HBox', null, VARIANT.NORMAL],
    textnormal: ['HBox'],
    textit: ['HBox', null, VARIANT.ITALIC],
    textbf: ['HBox', null, VARIANT.BOLD],
    textsf: ['HBox', null, VARIANT.SANSSERIF],
    texttt: ['HBox', null, VARIANT.MONOSPACE],
    tiny: ['SetSize', 0.5],
    Tiny: ['SetSize', 0.6],
    scriptsize: ['SetSize', 0.7],
    small: ['SetSize', 0.85],
    normalsize: ['SetSize', 1.0],
    large: ['SetSize', 1.2],
    Large: ['SetSize', 1.44],
    LARGE: ['SetSize', 1.73],
    huge: ['SetSize', 2.07],
    Huge: ['SetSize', 2.49],
    arcsin: 'NamedFn',
    arccos: 'NamedFn',
    arctan: 'NamedFn',
    arg: 'NamedFn',
    cos: 'NamedFn',
    cosh: 'NamedFn',
    cot: 'NamedFn',
    coth: 'NamedFn',
    csc: 'NamedFn',
    deg: 'NamedFn',
    det: 'NamedOp',
    dim: 'NamedFn',
    exp: 'NamedFn',
    gcd: 'NamedOp',
    hom: 'NamedFn',
    inf: 'NamedOp',
    ker: 'NamedFn',
    lg: 'NamedFn',
    lim: 'NamedOp',
    liminf: ['NamedOp', 'lim&thinsp;inf'],
    limsup: ['NamedOp', 'lim&thinsp;sup'],
    ln: 'NamedFn',
    log: 'NamedFn',
    max: 'NamedOp',
    min: 'NamedOp',
    Pr: 'NamedOp',
    sec: 'NamedFn',
    sin: 'NamedFn',
    sinh: 'NamedFn',
    sup: 'NamedOp',
    tan: 'NamedFn',
    tanh: 'NamedFn',
    limits: ['Limits', 1],
    nolimits: ['Limits', 0],
    overline: ['UnderOver', '2015'],
    underline: ['UnderOver', '2015'],
    overbrace: ['UnderOver', '23DE', 1],
    underbrace: ['UnderOver', '23DF', 1],
    overparen: ['UnderOver', '23DC'],
    underparen: ['UnderOver', '23DD'],
    overrightarrow: ['UnderOver', '2192'],
    underrightarrow: ['UnderOver', '2192'],
    overleftarrow: ['UnderOver', '2190'],
    underleftarrow: ['UnderOver', '2190'],
    overleftrightarrow: ['UnderOver', '2194'],
    underleftrightarrow: ['UnderOver', '2194'],
    overset: 'Overset',
    underset: 'Underset',
    overunderset: 'Overunderset',
    stackrel: ['Macro', '\\mathrel{\\mathop{#2}\\limits^{#1}}', 2],
    stackbin: ['Macro', '\\mathbin{\\mathop{#2}\\limits^{#1}}', 2],
    over: 'Over',
    overwithdelims: 'Over',
    atop: 'Over',
    atopwithdelims: 'Over',
    above: 'Over',
    abovewithdelims: 'Over',
    brace: ['Over', '{', '}'],
    brack: ['Over', '[', ']'],
    choose: ['Over', '(', ')'],
    frac: 'Frac',
    sqrt: 'Sqrt',
    root: 'Root',
    uproot: ['MoveRoot', 'upRoot'],
    leftroot: ['MoveRoot', 'leftRoot'],
    left: 'LeftRight',
    right: 'LeftRight',
    middle: 'LeftRight',
    llap: 'Lap',
    rlap: 'Lap',
    raise: 'RaiseLower',
    lower: 'RaiseLower',
    moveleft: 'MoveLeftRight',
    moveright: 'MoveLeftRight',
    ',': ['Spacer', MATHSPACE.thinmathspace],
    ':': ['Spacer', MATHSPACE.mediummathspace],
    '>': ['Spacer', MATHSPACE.mediummathspace],
    ';': ['Spacer', MATHSPACE.thickmathspace],
    '!': ['Spacer', MATHSPACE.negativethinmathspace],
    enspace: ['Spacer', .5],
    quad: ['Spacer', 1],
    qquad: ['Spacer', 2],
    thinspace: ['Spacer', MATHSPACE.thinmathspace],
    negthinspace: ['Spacer', MATHSPACE.negativethinmathspace],
    '*': 'DiscretionaryTimes',
    allowbreak: 'AllowBreak',
    goodbreak: ['Linebreak', TexConstant.LineBreak.GOODBREAK],
    badbreak: ['Linebreak', TexConstant.LineBreak.BADBREAK],
    nobreak: ['Linebreak', TexConstant.LineBreak.NOBREAK],
    break: 'Break',
    hskip: 'Hskip',
    hspace: 'Hskip',
    kern: ['Hskip', true],
    mskip: 'Hskip',
    mspace: 'Hskip',
    mkern: ['Hskip', true],
    rule: 'rule',
    Rule: ['Rule'],
    Space: ['Rule', 'blank'],
    nonscript: 'Nonscript',
    big: ['MakeBig', TEXCLASS.ORD, 0.85],
    Big: ['MakeBig', TEXCLASS.ORD, 1.15],
    bigg: ['MakeBig', TEXCLASS.ORD, 1.45],
    Bigg: ['MakeBig', TEXCLASS.ORD, 1.75],
    bigl: ['MakeBig', TEXCLASS.OPEN, 0.85],
    Bigl: ['MakeBig', TEXCLASS.OPEN, 1.15],
    biggl: ['MakeBig', TEXCLASS.OPEN, 1.45],
    Biggl: ['MakeBig', TEXCLASS.OPEN, 1.75],
    bigr: ['MakeBig', TEXCLASS.CLOSE, 0.85],
    Bigr: ['MakeBig', TEXCLASS.CLOSE, 1.15],
    biggr: ['MakeBig', TEXCLASS.CLOSE, 1.45],
    Biggr: ['MakeBig', TEXCLASS.CLOSE, 1.75],
    bigm: ['MakeBig', TEXCLASS.REL, 0.85],
    Bigm: ['MakeBig', TEXCLASS.REL, 1.15],
    biggm: ['MakeBig', TEXCLASS.REL, 1.45],
    Biggm: ['MakeBig', TEXCLASS.REL, 1.75],
    mathord: ['TeXAtom', TEXCLASS.ORD],
    mathop: ['TeXAtom', TEXCLASS.OP],
    mathopen: ['TeXAtom', TEXCLASS.OPEN],
    mathclose: ['TeXAtom', TEXCLASS.CLOSE],
    mathbin: ['TeXAtom', TEXCLASS.BIN],
    mathrel: ['TeXAtom', TEXCLASS.REL],
    mathpunct: ['TeXAtom', TEXCLASS.PUNCT],
    mathinner: ['TeXAtom', TEXCLASS.INNER],
    vtop: ['VBox', 'top'],
    vcenter: ['VBox', 'center'],
    vbox: ['VBox', 'bottom'],
    hsize: 'Hsize',
    parbox: 'ParBox',
    breakAlign: 'BreakAlign',
    buildrel: 'BuildRel',
    hbox: ['HBox', 0],
    text: 'HBox',
    mbox: ['HBox', 0],
    fbox: 'FBox',
    boxed: ['Macro', '\\fbox{$\\displaystyle{#1}$}', 1],
    framebox: 'FrameBox',
    makebox: 'MakeBox',
    strut: 'Strut',
    mathstrut: ['Macro', '\\vphantom{(}'],
    phantom: 'Phantom',
    vphantom: ['Phantom', 1, 0],
    hphantom: ['Phantom', 0, 1],
    smash: 'Smash',
    acute: ['Accent', '00B4'],
    grave: ['Accent', '0060'],
    ddot: ['Accent', '00A8'],
    dddot: ['Accent', '20DB'],
    ddddot: ['Accent', '20DC'],
    tilde: ['Accent', '007E'],
    bar: ['Accent', '00AF'],
    breve: ['Accent', '02D8'],
    check: ['Accent', '02C7'],
    hat: ['Accent', '005E'],
    vec: ['Accent', '2192'],
    dot: ['Accent', '02D9'],
    widetilde: ['Accent', '007E', 1],
    widehat: ['Accent', '005E', 1],
    matrix: 'Matrix',
    array: 'Matrix',
    pmatrix: ['Matrix', '(', ')'],
    cases: ['Matrix', '{', '', 'left left', null, '.2em', null,
        true],
    eqalign: ['Matrix', null, null, 'right left',
        THICKMATHSPACE, '.5em', 'D'],
    displaylines: ['Matrix', null, null, 'center', null, '.5em', 'D'],
    cr: 'Cr',
    '\\': 'CrLaTeX',
    newline: ['CrLaTeX', true],
    hline: ['HLine', 'solid'],
    hdashline: ['HLine', 'dashed'],
    eqalignno: ['Matrix', null, null, 'right left',
        THICKMATHSPACE, '.5em', 'D', null,
        'right'],
    leqalignno: ['Matrix', null, null, 'right left',
        THICKMATHSPACE, '.5em', 'D', null,
        'left'],
    hfill: 'HFill',
    hfil: 'HFill',
    hfilll: 'HFill',
    bmod: ['Macro', '\\mmlToken{mo}[lspace="' + THICKMATHSPACE + '"' +
            ' rspace="' + THICKMATHSPACE + '"]{mod}'],
    pmod: ['Macro', '\\pod{\\mmlToken{mi}{mod}\\kern 6mu #1}', 1],
    mod: ['Macro', '\\mathchoice{\\kern18mu}{\\kern12mu}' +
            '{\\kern12mu}{\\kern12mu}\\mmlToken{mi}{mod}\\,\\,#1',
        1],
    pod: ['Macro', '\\mathchoice{\\kern18mu}{\\kern8mu}' +
            '{\\kern8mu}{\\kern8mu}(#1)', 1],
    iff: ['Macro', '\\;\\Longleftrightarrow\\;'],
    skew: ['Macro', '{{#2{#3\\mkern#1mu}\\mkern-#1mu}{}}', 3],
    pmb: ['Macro', '\\rlap{#1}\\kern1px{#1}', 1],
    TeX: ['Macro', 'T\\kern-.14em\\lower.5ex{E}\\kern-.115em X'],
    LaTeX: ['Macro', 'L\\kern-.325em\\raise.21em' +
            '{\\scriptstyle{A}}\\kern-.17em\\TeX'],
    not: 'Not',
    dots: 'Dots',
    space: 'Tilde',
    '\u00A0': 'Tilde',
    ' ': 'Tilde',
    begin: 'BeginEnd',
    end: 'BeginEnd',
    label: 'HandleLabel',
    ref: 'HandleRef',
    nonumber: 'HandleNoTag',
    newcolumntype: 'NewColumnType',
    mathchoice: 'MathChoice',
    mmlToken: 'MmlToken'
}, BaseMethods);
new sm.EnvironmentMap('environment', ParseMethods.environment, {
    displaymath: ['Equation', null, false],
    math: ['Equation', null, false, false],
    array: ['AlignedArray'],
    darray: ['AlignedArray', null, 'D'],
    equation: ['Equation', null, true],
    eqnarray: ['EqnArray', null, true, true, 'rcl', 'bmt',
        ParseUtil.cols(0, MATHSPACE.thickmathspace), '.5em'],
    indentalign: ['IndentAlign']
}, BaseMethods);
new sm.CharacterMap('not_remap', null, {
    '\u2190': '\u219A',
    '\u2192': '\u219B',
    '\u2194': '\u21AE',
    '\u21D0': '\u21CD',
    '\u21D2': '\u21CF',
    '\u21D4': '\u21CE',
    '\u2208': '\u2209',
    '\u220B': '\u220C',
    '\u2223': '\u2224',
    '\u2225': '\u2226',
    '\u223C': '\u2241',
    '\u007E': '\u2241',
    '\u2243': '\u2244',
    '\u2245': '\u2247',
    '\u2248': '\u2249',
    '\u224D': '\u226D',
    '\u003D': '\u2260',
    '\u2261': '\u2262',
    '\u003C': '\u226E',
    '\u003E': '\u226F',
    '\u2264': '\u2270',
    '\u2265': '\u2271',
    '\u2272': '\u2274',
    '\u2273': '\u2275',
    '\u2276': '\u2278',
    '\u2277': '\u2279',
    '\u227A': '\u2280',
    '\u227B': '\u2281',
    '\u2282': '\u2284',
    '\u2283': '\u2285',
    '\u2286': '\u2288',
    '\u2287': '\u2289',
    '\u22A2': '\u22AC',
    '\u22A8': '\u22AD',
    '\u22A9': '\u22AE',
    '\u22AB': '\u22AF',
    '\u227C': '\u22E0',
    '\u227D': '\u22E1',
    '\u2291': '\u22E2',
    '\u2292': '\u22E3',
    '\u22B2': '\u22EA',
    '\u22B3': '\u22EB',
    '\u22B4': '\u22EC',
    '\u22B5': '\u22ED',
    '\u2203': '\u2204'
});
//# sourceMappingURL=BaseMappings.js.map